#!/bin/bash

# Get path of BayesWaveBurst to figure out where the postprocessing scripts are
pathtoBWB=$(which BayesWaveBurst)
# The argument for this script is the path to the job we want to analyze
dirname=$1
# The user may provide the sky-location (right ascension "ra" and declination "dec") of an injected
# signal, a request to compute matches between injected and reconsturcted signals, none of the above,
# or all of the above
echo
if [ "$#" -eq 3 ] ; then
    echo "You requested to display the sky location of the injections"
    ra=$2
    dec=$3
fi
if [ "$#" -eq 2 ] ; then
    echo "You provided an MDC log file"
    mdc=$2
fi

if [ -s "$pathtoBWB" ] ; then
    pathtoBWB=${pathtoBWB%src*}
    echo
    echo "Entering $dirname ..."
    cd $dirname
    MDC="$(grep MDC *bayeswave.run)"
    inj="$(grep \\.xml *bayeswave.run)"
    echo
    echo " * Running skymap..."
    if [ ${#MDC} -ne "0" -o ${#inj} -ne "0" ] ; then
    	#$pathtoBWB/postprocess/run_skymap.sh $ra $dec
        if [ "$#" -eq 3 ] ; then
    	    python $pathtoBWB/postprocess/skymap/megasky.py --ra=$ra --dec=$dec
        elif [ "$#" -eq 2 ] ; then
    	    python $pathtoBWB/postprocess/skymap/megasky.py --mdc=$mdc
        else
            echo "   Failed to generate skymap!"
        fi
    else
        python $pathtoBWB/postprocess/skymap/megasky.py
    fi
    echo
    echo " * Running megaplot..."
    python -W ignore $pathtoBWB/postprocess/megaplot.py
    #echo
    #echo " * Running plot_evidence..."
    #python -W ignore $pathtoBWB/postprocess/plot_evidence.py
    #echo
    #echo " * Running skyview..."
    #export PYTHONPATH=$PYTHONPATH:$pathtoBWB/postprocess/skymap:/home/jkanner/lib64/python:/home/jkanner/lib/python:$pathtoBWB/postprocess/s6
    #python -W ignore $pathtoBWB/postprocess/skyview.py
    echo
    echo " * Compressing analyzed data..."
    #tar -cf waveforms.tar waveforms
    #tar -cf chains.tar chains
    #tar -cf snr.tar *snr
    cd ..
    echo
    echo " * ...DONE!"
    echo
    echo
    echo
    echo "To move the postprocessing results in order to make them visible. Run:"
    echo "rsync -av --progress <path to this directory> <path to the desired directory in the public_html> --exclude '*/*snr' --exclude '*/waveforms' --exclude '*/chains' --exclude '*/post' --include='*/' --include='*html' --include '*/html/*'  --include '*/plots/*' --include '*/tables/*' --include condorOut*txt --include *bayeswave.run --exclude='*'"
    echo
    echo
else
    echo "Make sure the BayesWaveBurst command is in your path before running this script!"
fi
