#!/bin/bash

# Get path of BayesWaveBurst to figure out where the postprocessing scripts are
pathtoBWB=$(which BayesWaveBurst)
# The argument for this script is the path to the job we want to analyze
dirname=$1
# The user may provide the sky-location (right ascension "ra" and declination "dec") of an injected
# signal, a request to compute matches between injected and reconsturcted signals, none of the above,
# or all of the above
echo
if [ "$#" -eq 2 ] ; then
    echo "You requested to display matches and injected signals"
    match=$2
fi
if [ "$#" -eq 3 ] ; then
    echo "You requested to display the sky location of the injections"
    ra=$2
    dec=$3
fi
if [ "$#" -eq 4 ] ; then
    echo "You requested to display matches, injected signals, and the sky location of the injections"
    ra=$2
    dec=$3
    match=$4
fi

if [ -s "$pathtoBWB" ] ; then
    pathtoBWB=${pathtoBWB%src*}
    echo
    echo "Entering $dirname ..."
    cd $dirname
    MDC="$(grep MDC *bayeswave.run)"
    if [ ${#MDC} -ne "0" ] ; then
        echo
        echo " * Running plot_waveform..."
	if [ "$match" = "True" ]; then
            python -W ignore $pathtoBWB/postprocess/plot_waveform.py --mdc=True --match=True
	else
	    python -W ignore $pathtoBWB/postprocess/plot_waveform.py --mdc=True
	fi
        #echo
        #echo " * Running plot_evidence..."
	#python -W ignore $pathtoBWB/postprocess/plot_evidence.py
    else
        echo
        echo " * Running plot_waveform..."
	if [ "$match" = "True" ]; then
	    python -W ignore $pathtoBWB/postprocess/plot_waveform.py --mdc=False --match=True
	else
            python -W ignore $pathtoBWB/postprocess/plot_waveform.py --mdc=False
	fi
        #echo
        #echo " * Running plot_evidence..."
	#python -W ignore $pathtoBWB/postprocess/plot_evidence.py
    fi
    echo
    echo " * Running skymap..."
    $pathtoBWB/postprocess/run_skymap.sh $ra $dec
    echo
    echo " * Compressing analyzed data..."
    tar -cf waveforms.tar waveforms
    tar -cf chains.tar chains
    tar -cf snr.tar *snr
    cd ..
    echo
    echo " * ...DONE!"
    echo
    echo
    echo
    echo "To move the postprocessing results in order to make them visible. Run:"
    echo "  rsync -av --progress <path to this directory> <path to the desired directory in the public_html>  --exclude '*/*snr' --exclude '*/waveforms' --exclude '*/chains' --exclude '*/post' --include='*/' --include '*html' --include '*png' --include condorOut_BWB.txt --exclude='*'"
    echo
    echo
else
    echo "Make sure the BayesWaveBurst command is in your path before running this script!"
fi
