import matplotlib
matplotlib.use('Agg')
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.mlab as mlab
import glob
import argparse
import os
import pwd


# -------------------------------------------------
# Define helper function to extract median waveform
# -------------------------------------------------
def get_waveform(fileList, ifo):
    if not len(fileList) == 0:
        count=0
        for filename in fileList:
            time, strain = np.loadtxt(filename, unpack=True)
            if count == 0:
                strain_master = np.zeros((200, strain.size))
            strain_master[count] = strain
            count += 1
        print "   "+str(count)+" waveforms found for IFO "+str(int(ifo)+1)
        median_waveform = np.median(strain_master, axis=0)

        down_vec = np.zeros(median_waveform.size)
        up_vec = np.zeros(median_waveform.size)
        for sample in range(median_waveform.size):
            vector = np.transpose(strain_master)[sample]
            sort_vec = np.sort(vector)
            up_quart = sort_vec[ int(0.84*sort_vec.size) ]
            down_quart = sort_vec[ int(0.16*sort_vec.size) ]
            down_vec[sample] = down_quart
            up_vec[sample] = up_quart
            
    return (time, median_waveform, up_vec, down_vec)


# -- Parse command line arguments
parser = argparse.ArgumentParser(description='Produce html page for a BayesWaveBurst run.')
parser.add_argument('--mdc', default=False, help='Is this this an MDC? True/False (Default is false)')
parser.add_argument('--match', default=False, help="Add injected-reconstructed match to the webpage (Defaulti is true)")
args = parser.parse_args()
mdc = args.mdc
match = args.match

# ---------------------------------------
# Extract information from bayeswave.run
# ---------------------------------------
jobname = glob.glob('*bayeswave.run')[0]
bayeswave = open(jobname, 'r')

ifoNames = []
for line in bayeswave:
    spl = line.split()
    if len(spl) < 1: continue
    del spl[-1]
    # Determine names of IFOs
    if spl[0] == 'Command' and spl[1] == 'line:':
        for index, splElement in enumerate(spl):
            if splElement == '--ifo':
                ifoNames.append(spl[index+1]) 
        continue         
    # Determine number of IFOs
    if ' '.join(spl) == 'number of detectors':
        spl = line.split()
        ifoNum = spl[3]
        continue         
    # Determine gps trigger time
    if ' '.join(spl) == 'trigger time (s)':
        spl = line.split()
        gps = spl[3]
        break
ifoNum = int(ifoNum)
if ifoNum == 1:
    ifoList = ['0']
elif ifoNum == 2:
    ifoList = ['0', '1']
elif ifoNum == 3:
    ifoList = ['0', '1', '2']

# ----------------------------------
# Determine name of job 
# ----------------------------------
jobname = jobname.split('_')
del jobname[-1]
if not len(jobname) == 0:
    jobname = '_'.join(jobname)+'_'
else:
    jobname = ''

# -------------------------------------------------
# Read in data and median waveform to get plot axis
# -------------------------------------------------
axisList = []
# Determine the last data frame filename: avoids assuming it is *_199.dat.*
frame_list = os.listdir('waveforms/')
data_frame_list = frame_list
for file in frame_list[:]:
    if not('clean_1_data_' in file and '.dat.0' in file):
        data_frame_list.remove(str(file))
last_data_frame = str(data_frame_list[-1].split('.0')[0])

for ifo in ifoList:
    # -- Read Data
    #d_t, d_s = np.loadtxt('waveforms/'+str(jobname)+'clean_1_data_199.dat.{0}'.format(ifo), unpack=True)
    d_t, d_s = np.loadtxt('waveforms/'+last_data_frame+'.{0}'.format(ifo), unpack=True)

    # -- Read Signal model
    fileList = glob.glob('waveforms/'+str(jobname)+'signal_1_wave_*.dat.{0}'.format(ifo))
    (time, median_waveform, up_vec, down_vec) = get_waveform(fileList, ifo)
    
    # -- Get axis info
    wave = up_vec
    # wave = median_waveform
    wave_max = np.maximum(wave.max(), d_s.max())
    sig_times = time[wave > 0.05*wave.max()]
    axisList.append([sig_times.min(), sig_times.max(), -wave_max*1.1, wave_max*1.1])

# -- Select Axis with biggest y-value
ymax = 0
for axcand in axisList:
    if axcand[3] > ymax:
        ymax = axcand[3]
        axwinner = axcand

#print "   The axis winner is", axwinner

for ifo in ifoList:
    # -- Read in signal model and data
    fileList = glob.glob('waveforms/'+str(jobname)+'signal_1_wave_*.dat.{0}'.format(ifo))
    (time, median_waveform, up_vec, down_vec) = get_waveform(fileList, ifo)
 
    # -- Read in the injected waveform
    if match=='True':
        fileList = glob.glob(str(jobname)+'post/injected_waveforms/'+str(jobname)+'signal_1_wave_*.dat.{0}'.format(ifo))
        (inj_time, inj_median_waveform, inj_up_vec, inj_down_vec) = get_waveform(fileList, ifo)

    # -- Read whitened data
    #d_t, d_s = np.loadtxt('waveforms/'+str(jobname)+'clean_1_data_199.dat.{0}'.format(ifo), unpack=True)
    d_t, d_s = np.loadtxt('waveforms/'+last_data_frame+'.{0}'.format(ifo), unpack=True)
  
    # -- Plot the whitened data, the injected signal (if asked for), and the median waveform
    fignum = plt.figure()
    plt.plot(d_t, d_s, 'k', linewidth=1, alpha=0.3)
    if match=='True':
        plt.plot(inj_time, inj_median_waveform, 'r', linewidth=1)
        plt.fill_between(inj_time, inj_down_vec, inj_up_vec, facecolor='red', edgecolor='red', alpha=0.2)
    plt.plot(time, median_waveform, 'b', linewidth=2)
    plt.fill_between(time, down_vec, up_vec, facecolor='blue', edgecolor='blue', alpha=0.3)
   

    axisList.append([sig_times.min(), sig_times.max(), -wave_max*1.1, wave_max*1.1])
    plt.xlabel('Time (s)')
    plt.ylabel('Best Fit Waveform (blue) in whitened data (red)')
    plt.title('IFO {0}'.format(ifo))

    # -- Save the full axis plot
    plt.savefig('waveform_{0}_full.png'.format(ifo))

    # -- Save the plot
    plt.axis(axwinner)
    plt.savefig('waveform_{0}.png'.format(ifo))
    plt.close()
    
    # -- Attempt to make a spectrogram of median waveform
    ts = time[1] - time[0]
    fs = int(np.round(1.0/ts))
    NFFT = fs / 16
    window = np.blackman(NFFT)
    holder = np.zeros(1*NFFT)
    seg = np.append(holder, median_waveform)
    seg = np.append(seg, holder)
    spec_power, freqs, bins = mlab.specgram(median_waveform, NFFT=NFFT, Fs=fs, 
                                            window=window, noverlap=int(0.8*NFFT))
    #plt.savefig('spec_{0}.png'.format(ifo))
    #plt.close()
    
    # -- Try eliminating quiet pixels
    sort_power = np.sort(spec_power.flatten())[::-1]
    sum_power = np.cumsum(sort_power)
    indx = np.nonzero(sum_power < 0.95*sum_power.max())
    point90 = sort_power[indx[0].max()]
    spec_power[ spec_power < point90 ] = spec_power.min()
    vmin = np.log10(spec_power[ spec_power > spec_power.min()].min())

    # -- Try do it yourself spectrogram
    fig = plt.figure()
    ax = plt.subplot(111)
    binwidth = (bins[1] - bins[0]) / 2.0
    # TODO: why was this no longer giving the right spectrogram? is the new version correct?
    #cax = ax.pcolormesh(bins-0.5-binwidth, freqs, np.log10(spec_power), vmin=vmin, cmap='OrRd')
    cax = ax.pcolormesh(bins-(time[-1]-time[0])/2.0-binwidth, freqs, np.log10(spec_power), vmin=vmin, cmap='OrRd')
    plt.title('IFO {0}'.format(ifo))
    plt.xlabel('Time (s)')
    plt.ylabel('Frequency (Hz)')
    plt.axis([axwinner[0], axwinner[1], 0, fs/2])
    #plt.axis([time[0], time[-1], 0, fs/2])
    cb = plt.colorbar(cax, orientation='horizontal')
    cb.set_label('Logrithmic Signal Power')
    plt.savefig('self_spec_{0}.png'.format(ifo))
    plt.close()

    # --------------------------------
    # -- Plot the median glitch model
    # --------------------------------
    try:
        fileList = glob.glob('waveforms/'+str(jobname)+'glitch_1_glitch_*.dat.{0}'.format(ifo))
        (time, median_waveform, up_vec, down_vec) = get_waveform(fileList, ifo)

        plt.figure()
        plt.plot(d_t, d_s, 'r', linewidth=1, alpha=0.3)
        plt.plot(time, median_waveform, 'k', linewidth=2)
        plt.fill_between(time, down_vec, up_vec, facecolor='black', edgecolor='black', alpha=0.3)
    
        # -- Adjust plot properties

        plt.xlabel('Time (s)')
        plt.ylabel('Best Fit Glitch Model (whitened)')
        plt.title('IFO {0}'.format(ifo))

        # -- Save the plot
        plt.savefig('glitch_{0}_full.png'.format(ifo))
        plt.axis(axwinner)
        plt.savefig('glitch_{0}.png'.format(ifo))
        plt.close()



    except:
        print "No glitch model"

    # -----------------------
    # Plot the median PSDs
    # -----------------------

    fileList = glob.glob('waveforms/'+str(jobname)+'clean_1_psd_*.dat.{0}'.format(ifo))

    if not len(fileList) == 0:
        count=0
        for filename in fileList:
            freq, power, spline, line = np.loadtxt(filename, unpack=True)
            if count == 0:
                power_master = np.zeros((200, power.size))
                spline_m = np.zeros((200, power.size))
                line_m = np.zeros((200, power.size))

            power_master[count] = power
            spline_m[count] = spline
            line_m[count] = line

            count += 1

        median_power = np.median(power_master, axis=0)
        median_line = np.median(line_m, axis=0)
        median_spline = np.median(spline_m, axis=0)

        plt.figure()
        plt.loglog(freq, median_power,'b', alpha=0.4, label='PSD of Noise')
        plt.loglog(freq, median_line + median_spline, 'r', label='Noise model')
        plt.xlabel('Frequency')
        plt.ylabel('PSD')
        plt.title('PSD for IFO {0}'.format(ifo))
        plt.xlim([freq.min(), freq.max()])
        plt.grid()
        plt.legend()
        plt.savefig('psd_{0}.png'.format(ifo))
        plt.close()

# -----------------------                                                                                                                                           
# For each detector plot the overlap and find its median value
# ----------------------- 
if match=='True':
    plt.figure()
    overlapList = []
    for ifo in ifoList:
        # -- Read the data 
        overlap_h, overlap_hp, overlap_hc = np.loadtxt(str(jobname)+'post/'+'overlap.dat.{0}'.format(ifo), unpack=True)
        # -- Plot the overlap data 
        plt.plot(overlap_h, linewidth=1, label=ifoNames[int(ifo)])
        # -- Get median match                                                                                                                                          
        overlap_h_reduced = overlap_h[1000:]
        median_overlap_h = np.median(overlap_h_reduced)
        overlapList.append(median_overlap_h)
        print str(ifoNames[int(ifo)]), ifo, str(median_overlap_h)
    plt.xlabel('MCMC iteration')
    plt.ylabel('Overlap')
    plt.title('Overlap in each IFO')
    plt.legend(loc=4)
    plt.savefig('overlap.png')
    plt.close()

# -----------------------                                                                                                                                           
# Plot the network overlap and find its median value
# ----------------------- 
if match=='True':
    # -- Read the data 
    network_overlap_h, network_overlap_hp, network_overlap_hc = np.loadtxt(str(jobname)+'post/'+str(jobname)+'network_overlap.dat', unpack=True)
    # -- Plot the overlap data 
    plt.figure()
    plt.plot(network_overlap_h, 'blue', linewidth=1, alpha=0.3)
    plt.xlabel('MCMC iteration')
    plt.ylabel('Normalised Network overlap')
    plt.title('Network overlap')
    plt.savefig('network_overlap.png')
    plt.close()
    # -- Get median match                                                                                                                                          
    network_array_reduced_h = network_overlap_h[1000:]
    network_median_overlap_h = np.median(network_array_reduced_h)

# --------------
# Plot Evidence
# --------------

maxodds=20

# -- Plotting helper function
def limit_odds(odds):
    if odds > maxodds:
        odds = 0.95*maxodds
    elif odds < -maxodds:
        odds = -0.95*maxodds
    return odds

try: 
    infile = open(str(jobname)+'evidence.dat', 'r')
    maxodds = 20
    for line in infile:
        spl = line.split()
        if spl[0] == 'noise':
            sig_noise = -float(spl[1])
        if spl[0] == 'glitch':
            sig_gl = -float(spl[1])
    infile.close()
except:
    print "Couldn't read evidence.dat"
    sig_noise = 0.0
    sig_gl = 0.0

print
print '   log( E_signal / E_noise ) =', sig_noise
print '   log( E_signal / E_glitch ) =', sig_gl

plt.figure()
# plt.plot(limit_odds(sig_gl), limit_odds(sig_noise), '*', markersize=15, mfc='yellow')
plt.plot(sig_gl, sig_noise, '*', markersize=15, mfc='yellow')
plt.ylabel('log( E_signal / E_noise )')
plt.xlabel('log( E_signal / E_glitch )')

maxodds = 1.1*np.array( [np.abs(sig_gl), np.abs(sig_noise), 20] ).max()
xmaxodds = 1.1*np.maximum(np.abs(sig_gl), 20)
ymaxodds = 1.1*np.maximum(np.abs(sig_noise), 20)

plt.axis([-xmaxodds, xmaxodds, -ymaxodds, ymaxodds])

plt.fill_between([0,maxodds], [0, 0], [maxodds, maxodds], facecolor='green', interpolate=True, alpha=0.3)
plt.fill_between([-maxodds,0,maxodds], [-maxodds,0,0], [-maxodds, -maxodds, -maxodds], facecolor='blue', interpolate=True, alpha=0.3)
plt.fill_between([-maxodds,0], [-maxodds, 0], [maxodds, maxodds], facecolor='red', interpolate=True, alpha=0.3)

plt.grid()

plt.text(0.9*xmaxodds, 0.9*ymaxodds, 'Signal',
        horizontalalignment='right',
        verticalalignment='top')

plt.text(-0.9*xmaxodds, 0.9*ymaxodds, 'Glitch',
        horizontalalignment='left',
        verticalalignment='top')

plt.text(0.9*xmaxodds, -0.9*ymaxodds, 'Noise',
        horizontalalignment='right',
        verticalalignment='bottom')

plt.savefig('odds.png')
plt.close()

# ----------------
# Plot Temp Chains
# ---------------
modelList = ('glitch', 'signal', 'noise')

# -- Make first plot
plt.figure()
for model in modelList:
    try:
        temp, likehood, varlikehood = np.recfromtxt(str(jobname)+"{0}_1_evidence.dat".format(model), unpack=True)
        plt.semilogx(temp, likehood, label=model, linewidth=2)
    except:
        pass
plt.xlabel( '1/Temp' )
plt.ylabel('log(L)')
plt.grid()
plt.legend(loc=2)
plt.savefig('likelihood.png'.format(model))
plt.close()

# -- Make 2nd plot
plt.figure()
for model in modelList:
    try:
        temp, likehood, varlikehood = np.recfromtxt(str(jobname)+"{0}_1_evidence.dat".format(model), unpack=True)
        plt.semilogx(temp, likehood*temp, linewidth=2, label=model)
    except:
        pass
plt.grid()
plt.xlabel('1/Temp')
plt.ylabel('log(L) X 1/Temp')
plt.legend(loc=2)
plt.savefig('TL.png'.format(model))
plt.close()

# ---------------------
# Make web page
# ---------------------
web = open('index.html', 'w')
web.write('<html><body>\n')

# -- Get some text from condor out: if condorOut.txt is not there, we will use the info from bayeswave.run
info = ""
if os.path.isfile('condorOut.txt'):
    condor = open('condorOut.txt', 'r')
    ifoNames = []
    for line in condor:
        spl = line.split()
        if len(spl) < 1: continue
        if spl[0] == 'Detector':
            ifoNames.append(spl[1].strip(':'))
        if spl[0] == 'trigger':
            gps = float(spl[3])
        if line.find('Run Flags') > 0:
            break
    #    if line.find('Run Flags') > 0:
    #        break
    #    else:
    #        info = info + line
    condor.close()

# -- If MDC, read SNR info
if mdc=='True':
    print "Looking for SNRs..."
    snrList = []
    foundSnr=False
    snrFile = open('condorOut.txt', 'r')
    for line in snrFile:
        if line.find('Recompute')>0:
            print "Found Recompute section!"
            foundSnr = True
        if (line.find('Injected SNR')>-1) and foundSnr:
            print "Found an SNR!!"
            snrList.append(float(line.split()[2]))
    snrFile.close


info = info + "The detectors are {0} \n".format( ', '.join(ifoNames) )
info = info + "   The trigger time is GPS {0} \n".format(gps)

# -- Start the web page
web.write('<h1>Bayeswave Burst Output page</h1>\n')
web.write("<h3>Results for trigger at GPS {0}</h3>".format(gps) )
web.write("<h3>Detector names: {0}</h3>".format(', '.join(ifoNames) ) )

# -- Print SNRs if MDC
if mdc=='True':
    web.write("<h3>Matched Filter SNRs of injections</h3>")
    for ifo, snr in zip(ifoNames, snrList):
        web.write("{0} injected with SNR {1:.3f}<br/>".format(ifo, snr))

# -- Write out the basic log info
web.write('<h3>Log Info</h3>')
# web.write("<pre>{0}</pre>".format(info))
web.write("<a href='./condorOut.txt'>See full log file</a>")
print "   Info: {0}".format(info)


# -- Write out the bayes Factor plot
web.write('<h3>Evidence for Signal</h3>')
web.write(" log(Evidence_signal / Evidence_glitch) = {0:.2f} <br/>".format(sig_gl) )
web.write(" log(Evidence_signal / Evidence_noise) = {0:.2f} <br/>".format(sig_noise) )
web.write("<img src='./odds.png' width=600 alt='odds.png'>")

# -- Write out the waveforms
web.write("<h3>Median Signal Model Waveforms and 1-sigma Uncertainties</h3>")
if match=='True':
    web.write("Reconstructed waveforms in whitened data.  Black is data, red is injected waveform, blue is recovered waveform<br/>")
else:
    web.write("Reconstructed waveforms in whitened data.  Black is data, blue is reconstructed waveform<br/>")
for ifo in ifoList:
    web.write("<img src='./waveform_{0}.png' alt='waveform_{0}.png' width=500>".format(ifo))

# -- Write out the overlap plots                                                                                                                                         
if match=='True':
    web.write('<h3>Match between injected and reconstructed waves</h3>')
    web.write("Match = {0:.3f} <br/>".format(network_median_overlap_h) )
    web.write("<img src='./network_overlap.png' width=600 alt='network_overlap.png'>")
    web.write("<br/>")
    web.write("<br/>")
    for ifo, median_overlap_h in zip(ifoNames, overlapList):
        web.write("Match in {0} = {1:.3f} <br/>".format(ifo, median_overlap_h) )
    web.write("<img src='./overlap.png' width=600 alt='overlap.png'>")

# -- Write out Glitch model
web.write("<h3>Median Glitch Model Waveforms and 1-sigma Uncertainties</h3>")
web.write("Reconstructed waveforms in whitened data<br/>")
for ifo in ifoList:
    web.write("<img src='./glitch_{0}.png' alt='glitch_{0}.png' width=500>".format(ifo))

# -- Write out spectrogram
web.write("<h3>Spectrogram of median reconstructed signal model waveform</h3>")
for ifo in ifoList:
    web.write("<img src='./self_spec_{0}.png' alt='self_spec_{0}.png' width=500>".format(ifo) )

# -- Write out the PSDs
web.write("<h3>PSDs and Reconstructed PSDs</h3>")
for ifo in ifoList:
    web.write("<img src='./psd_{0}.png' alt='psd_{0}.png' width=500>".format(ifo))

# -- Write out diagnostic plots
web.write("<h3>Diagnostic plots of Likelihood and Temperature</h3>")
web.write("<img src='./likelihood.png' alt='likelihood.png' width=500>")
web.write("<img src='./TL.png' alt='TL.png' width=500> <br/>" )
    
# -- Include skymap
web.write('<h3>Skymap</h3>')
web.write("<img src='./skymap.png' alt='No_skymap_available' width=650>")

#-- Close the web site
web.write('</body></html>')
web.close()


