import matplotlib
matplotlib.use('Agg')
import numpy as np
import matplotlib.pyplot as plt
import glob
import os
import sys
from pylal.xlal.datatypes.ligotimegps import LIGOTimeGPS
from pylal.xlal.date import XLALGreenwichSiderealTime
import find_trig as ft
import healpy as hp
from subprocess import call
# from scipy.optimize import curve_fit
# import acor
# from pylal import bayespputils
# import sky_area.sky_area_clustering as sky
# import skyview
from scipy import stats
import readbwb

# --------------
# Parameters
# --------------
mdcfile = '/home/jkanner/baysewave/svn/trunk/burstinj/s6/BurstMDC-BRST_S6-Log.txt'

# -- Close any open figures
plt.close('all')

# ----------------
# Read S6 MDC log file
# -----------------
# print "Reading MDC log (this may take a few minutes) ..."
# mdc = ft.Mdc(filename=mdcfile)
# print "I read the mdc log!"


# --------------------------------
# Helper function to read mode files
# ---------------------------------
def read_mode(filename):
    data = np.loadtxt(filename)
    # -- Format of returned arrays are:
    #    mode, low, high, injected
    #    low and high correspond to 90% confidence
    overlap = np.append(data[0], 1.0)
    energy  = np.append(data[1], data[2][0])
    t0      = np.append(data[3], data[4][0])
    dur     = np.append(data[5], data[6][0])
    f0      = np.append(data[7], data[8][0])
    band    = np.append(data[9], data[10][0])
            
    return(overlap, energy, t0, dur, f0, band)



# -- Get directory list
topdir = os.getcwd()
dirList = glob.glob('*/job*')
# dirList = [os.path.abspath(directory) for directory in dirList]

# -- Loop over jobs
ifoList = ['0', '1']
whiteList = [False, True]
for ifo in ifoList:
    for white in whiteList:
        # -- Set up lists of results
        overlap = []
        energy = []
        t0 = []
        duration = []
        f0 = []
        band = []
        scale = []
        snr = []
        snr0 = []
        snr1 = []
        goodJobList = []
        gps = []
    
        for job in dirList:
            os.chdir(job)
            print "Entered", os.getcwd()
            jobname = glob.glob('*bayeswave.run')[0]
            bayeswave = open(jobname, 'r')

            # ---------------------------------------
            # Extract job parameters
            # ---------------------------------------
            param = readbwb.BwbParams()
            scale.append(param.scale)
            # ifoNum = param.numifo
            # gps = param.gps
            gps.append(param.gps)

            # -- Add the network snr to the list
            snrList = param.snr
            snr0.append(snrList[0])
            snr1.append(snrList[1])
            sqsnr = snrList**2
            netsnr = np.sqrt(sqsnr.sum())
            snr.append(netsnr)

            # ---------
            # Read modes
            # ---------
            if white:
                ov, en, t, dur, f, b = read_mode('w_mode_{0}.txt'.format(ifo))
            else:
                ov, en, t, dur, f, b = read_mode('mode_{0}.txt'.format(ifo))
            overlap.append(ov)
            energy.append(en)
            t0.append(t)
            duration.append(dur)
            f0.append(f)
            band.append(b)
                                                             
            goodJobList.append(job)

            # -- Return to top level directory, continue through list of jobs
            plt.close('all')
            os.chdir(topdir)

        # -- Output file for Tyson
        if white:
            tyson = open('w_plot_me_{0}.txt'.format(ifo), 'w')
        else:
            tyson = open('plot_me_{0}.txt'.format(ifo), 'w')
        tyson.write("# GPS SNR_net SNR_0 SNR_1 [overlap] [energy] [t0] [duration] [f0] [bandwidth]\n")
        tyson.write("# [] means [mode low high injected], where low and high correspond to the 90% confidence bound\n")
        for i in range(len(overlap)):
            tyson.write("{0} {1} {2} {3} {4}".format(gps[i], snr[i], snr0[i], snr1[i], str(overlap[i])[1:-1] ))
            tyson.write("{0} {1} {2} {3} {4}\n".format(str(energy[i])[1:-1], str(t0[i])[1:-1], str(duration[i])[1:-1], str(f0[i])[1:-1], 
                                                       str(band[i])[1:-1] ))
        tyson.close()
    
        # ----------------------
        # Make histogram of SNRs
        # ----------------------
        plt.figure()
        plt.hist(snr)
        plt.xlabel('SNR')
        plt.ylabel('Number')
        plt.savefig('snr_hist_{0}.png'.format(ifo))
        plt.close()

        # -------------------
        # Plot central freq
        # ------------------
        centralfreq = f0
        f0 = np.array(f0).transpose()
        y = f0[0]
        ylow = y - f0[1]
        yhigh = f0[2] - y
        inj  = f0[3]
        x = snr
        fig = plt.figure()
        plt.plot(snr, inj, 'r.', alpha=0.5)
        plt.errorbar(snr, y, yerr=(ylow, yhigh), fmt='.')        
        plt.xlabel('Network SNR')
        plt.ylabel('Central Frequency (Hz)')
        plt.title('IFO {0}'.format(ifo))
        if white:
            plt.savefig('w_freq_summary_{0}.png'.format(ifo))
        else:
            plt.savefig('freq_summary_{0}.png'.format(ifo))
        plt.close()


        # -------------------
        # Plot Bandwidth
        # ------------------
        f0 = np.array(band).transpose()
        y = f0[0]
        ylow = y - f0[1]
        yhigh = f0[2] - y
        inj  = f0[3]
        x = snr
        plt.figure()
        plt.errorbar(snr, y, yerr=(ylow, yhigh), fmt='.', label='90% Confidence')
        plt.plot(snr, inj, 'r.', alpha=0.5, label='Injected')
        plt.xlabel('Network SNR')
        plt.ylabel('Bandwidth (Hz)')
        plt.title('IFO {0}'.format(ifo))
        if white:
            plt.savefig('w_band_summary_{0}.png'.format(ifo))
        else:
            plt.savefig('band_summary_{0}.png'.format(ifo))
        plt.close()

        # -------------------
        # Plot central time
        # ------------------
        f0 = np.array(t0).transpose()
        y = f0[0]
        ylow = y - f0[1]
        yhigh = f0[2] - y
        inj  = f0[3]
        x = snr
        plt.figure()
        plt.errorbar(snr, y, yerr=(ylow, yhigh), fmt='.', label='90% Confidence')
        plt.plot(snr, inj, 'r.', label='Injected')
        plt.xlabel('Network SNR')
        plt.ylabel('Central Time')
        plt.title('IFO {0}'.format(ifo))
        plt.legend()
        if white:
            plt.savefig('w_time_summary_{0}.png'.format(ifo))
        else:
            plt.savefig('time_summary_{0}.png'.format(ifo))
        plt.close()

        # ------------------
        # Plot duration
        # ------------------
        f0 = np.array(duration).transpose()
        f0 = 10**f0
        y = f0[0]
        ylow = y - f0[1]
        yhigh = f0[2] - y
        inj  = f0[3]
        x = snr
        plt.figure()
        plt.errorbar(snr, y, yerr=(ylow, yhigh), fmt='.', label='90% Confidence')
        plt.plot(snr, inj, 'r.', label='Injected')
        plt.xlabel('Network SNR')
        plt.ylabel('Duration (s)')
        plt.title('IFO {0}'.format(ifo))
        plt.legend()
        if white:
            plt.savefig('w_dur_summary_{0}.png'.format(ifo))
        else:
            plt.savefig('dur_summary_{0}.png'.format(ifo))
        plt.close()

        # ------------------
        # Plot Signal Energy
        # ------------------
        f0 = np.array(energy).transpose()
        f0 = 10**f0
        y = f0[0]
        ylow = y - f0[1]
        yhigh = f0[2] - y
        inj  = f0[3]
        x = snr
        plt.figure()
        plt.errorbar(snr, y, yerr=(ylow, yhigh), fmt='.', label='90% Confidence')
        plt.plot(snr, inj, 'r.', label='Injected')
        plt.xlabel('Network SNR')
        plt.ylabel('Signal Energy')
        plt.title('IFO {0}'.format(ifo))
        plt.legend()
        if white:
            plt.savefig('w_energy_summary_{0}.png'.format(ifo))
        else:
            plt.savefig('energy_summary_{0}.png'.format(ifo))
        plt.close()

        # -------------------
        # Plot overlap
        # ------------------
        f0 = np.array(overlap).transpose()
        y = f0[0]
        ylow = y - f0[1]
        yhigh = f0[2] - y
        inj  = f0[3]
        x = snr
        plt.figure()
        plt.errorbar(snr, y, yerr=(ylow, yhigh), fmt='.', label='90% Confidence')
        plt.plot(snr, inj, 'r.', label='Injected')
        plt.xlabel('Network SNR')
        plt.ylabel('Overlap between injected and reconstructed signal')
        plt.title('IFO {0}'.format(ifo))
        plt.legend()
        if white:
            plt.savefig('w_overlap_summary_{0}.png'.format(ifo))
        else:
            plt.savefig('overlap_summary_{0}.png'.format(ifo))
        plt.close()




# -----------------------
# Make top level web page
# ----------------------
outfile = open('pec_top.html', 'w')
web = outfile
outfile.write('<html><body>')
outfile.write('<h1>BayesWave Burst PEC Summary Page</h1>')
web.write('<ul>')
web.write('<li> For a description of this study, see the <a href=https://wiki.ligo.org/Bursts/BayesWave/BwbPecInj>WIKI PAGE</a>')
web.write('<li> Each point representes one injection')
web.write('<li> Blue points are mode in posterior (i.e. most likely) for recovered signal')
web.write('<li> Blue error bars are 90% confidendence interal for recovered signal')
web.write('<li> Red points are the injected values.')
web.write('<li> In some cases, the confidence region in the overlap seems to extend beyond 1 due to a binning issue')
web.write('</ul>')
web.write('<h2>Plots in Colored Data</h2>')
outfile.write("<h3>Overlap</h3>")
web.write("The normalized inner product ( rec  |  inj ) for the recovered and injected waveforms.  Not weighted by noise power.<br/>")
for ifo in ifoList:
    outfile.write('<img src=overlap_summary_{0}.png width=500>'.format(ifo))
web.write("<h3>Central Time</h3>")
web.write("The central time of the recovered signal <br/>")
for ifo in ifoList:
    outfile.write('<img src=time_summary_{0}.png width=500>'.format(ifo))
web.write("<h3>Duration</h3>")
for ifo in ifoList:
    outfile.write('<img src=dur_summary_{0}.png width=500>'.format(ifo))
web.write("<h3>Central Frequency</h3>")
for ifo in ifoList:
    outfile.write('<img src=freq_summary_{0}.png width=500>'.format(ifo))
web.write("<h3>Bandwidth</h3>")
for ifo in ifoList:
    outfile.write('<img src=band_summary_{0}.png width=500>'.format(ifo))
web.write("<h3>Signal Energy</h3>")
web.write("May be improperly normalized. <br/>")
for ifo in ifoList:
    outfile.write('<img src=energy_summary_{0}.png width=500>'.format(ifo))

web.write('<h2>Plots in Whitened Data</h2>')
outfile.write("<h3>Overlap</h3>")
web.write("The normalized inner product ( rec  |  inj ) for the recovered and injected waveforms.  Weighted by PSD. <br/>")
for ifo in ifoList:
    outfile.write('<img src=w_overlap_summary_{0}.png width=500>'.format(ifo))
web.write("<h3>Central Time</h3>")
web.write("The central time of the recovered signal <br/>")
for ifo in ifoList:
    outfile.write('<img src=w_time_summary_{0}.png width=500>'.format(ifo))
web.write("<h3>Duration</h3>")
for ifo in ifoList:
    outfile.write('<img src=w_dur_summary_{0}.png width=500>'.format(ifo))
web.write("<h3>Central Frequency</h3>")
for ifo in ifoList:
    outfile.write('<img src=w_freq_summary_{0}.png width=500>'.format(ifo))
web.write("<h3>Bandwidth</h3>")
for ifo in ifoList:
    outfile.write('<img src=w_band_summary_{0}.png width=500>'.format(ifo))
web.write("<h3>Signal Energy</h3>")
web.write("May be improperly normalized. <br/>")
for ifo in ifoList:
    outfile.write('<img src=w_energy_summary_{0}.png width=500>'.format(ifo))




# --------------------------------
# Try making sorted list of injections
# -----------------------------

# -- Make tuple, sort by SNR
f0 = np.array(overlap).transpose()
mode = f0[0]
low  = f0[1]

raw_eventList = zip(snr, mode, low, goodJobList)
eventList =  sorted(raw_eventList, key=lambda event: event[0], reverse=True)

web.write("<h2>Table sorted by SNR</h2>")
web.write('<table>')
web.write('<tr><b><td>SNR <td>Job    <td>Overlap (mode) <td>Overlap (min) </b></tr>')
for (snr, mode, low, job) in eventList:
    outfile.write("""<tr>
<td>{0:.2f}  
<td><a href='{1}/pec.html'>{1}</a>  
<td>{2:.2f}  
<td>{3:.2f}
</tr>
""".format(snr, job, mode, low) )
outfile.write('</table></html></body>')

outfile.close()
