import matplotlib
matplotlib.use('Agg')
import numpy as np
import matplotlib.pyplot as plt
import glob
import os
import sys
from pylal.xlal.datatypes.ligotimegps import LIGOTimeGPS
from pylal.xlal.date import XLALGreenwichSiderealTime
import find_trig as ft
import healpy as hp
from subprocess import call
# from scipy.optimize import curve_fit
import acor
from pylal import bayespputils
import sky_area.sky_area_clustering as sky
import skyview
from scipy import stats

# --------------
# Parameters
# --------------
# -- Set skymap resolution
NSIDE = 32
mdcfile = '/home/jkanner/baysewave/svn/trunk/burstinj/s6/BurstMDC-BRST_S6-Log.txt'

# -- Close any open figures
plt.close('all')

# ----------------
# Read S6 MDC log file
# -----------------
print "Reading MDC log (this may take a few minutes) ..."
mdc = ft.Mdc(filename=mdcfile)
print "I read the mdc log!"


# -- Set up lists of results
glitch = []
signal = []
scale = []
snrList = []
goodJobList = []
skydict = {}
# injcontourList = []
# searchareaList = []
# area50List = []
# area90List = []

# -- Get directory list
topdir = os.getcwd()
dirList = glob.glob('*/job*')
# dirList = [os.path.abspath(directory) for directory in dirList]

# -- Loop over jobs
for job in dirList:
    os.chdir(job)
    print "Entered", os.getcwd()
    jobname = glob.glob('*bayeswave.run')[0]
    bayeswave = open(jobname, 'r')

    # ---------------------------------------
    # Extract information from bayeswave.run
    # ---------------------------------------
    ifoNames = []
    for line in bayeswave:
        spl = line.split()
        if len(spl) < 1: continue
        
        # Determine names of IFOs and MDC scale factor
        if spl[0] == 'Command' and spl[1] == 'line:':
            for index, splElement in enumerate(spl):
                if splElement == '--ifo':
                    ifoNames.append(spl[index+1]) 
                if splElement == '--MDC-prefactor':
                    print spl
                    scale.append(float(spl[index+1]))
                    print "At this point, scale is {0}".format(scale)
            continue         

        del spl[-1]
        # Determine number of IFOs
        if ' '.join(spl) == 'number of detectors':
            spl = line.split()
            ifoNum = spl[3]
            continue         
        # Determine gps trigger time
        if ' '.join(spl) == 'trigger time (s)':
            spl = line.split()
            gps = spl[3]
            break
    ifoNum = int(ifoNum)
    if ifoNum == 1:
        ifoList = ['0']
    elif ifoNum == 2:
        ifoList = ['0', '1']
    elif ifoNum == 3:
        ifoList = ['0', '1', '2']
    
    # ----------------------------------
    # Determine name of job 
    # ----------------------------------
    jobname = jobname.split('_')
    del jobname[-1]
    if not len(jobname) == 0:
        jobname = '_'.join(jobname)+'_'
    else:
        jobname = ''

    # -------------------
    # Read evidence file
    # -------------------
    try:
        infile = open(str(jobname)+'evidence.dat', 'r')
    except:
        os.chdir(topdir)
        continue
    
    for line in infile:
        spl = line.split()
        if spl[0] == 'noise':
            sig_noise = -float(spl[1])
            signal.append(sig_noise)
        if spl[0] == 'glitch':
            sig_gl = -float(spl[1])
            glitch.append(sig_gl)
    infile.close()

    # -- Parse log file
    infile = open('condorOut.txt', 'r')
    redo = False
    snr = []
    print "Parsing log file..."
    for line in infile:
        if line.find('Recompute') > 0:
            print "Found recompute line"
            print line
            redo = True
        if redo:
            if line.find('Injected SNR') > -1:
                print "Found line with SNR"
                print line
                snr.append(float(line.split()[2]))
    if len(snr) == 0:
        sys.exit("No injections found in this run!")
    snrList.append(np.sqrt(snr[0]**2 + snr[1]**2 ))
    goodJobList.append(job)
    infile.close()

    # --------------------------------
    # Process skymap information
    # --------------------------------
    # -- Check if skystats exists
    if os.path.isfile('skystats.txt'):
        # area50, area90, searcharea, injcontour = np.loadtxt('skystats.txt', unpack=True)
        print "Found skystats file!"
        # skydata = np.recfromtxt('skystats.txt', names='True')
        # for key in ['area50', 'area90', 'searcharea', 'injcontour']:
        #    skydict[key].append(skydata[key])

        skydata = open('skystats.txt', 'r')
        keylist = skydata.readline().split()[1:]
        valuelist = skydata.readline().split()
        for key, value in zip(keylist, valuelist):
            if key not in skydict: skydict[key] = []
            skydict[key].append(float(value))
    else:
        print "No skystats found :(  Running skyview.  You have time for a coffee break ..."
        skyview.make_skyview(mdc=mdc, NSIDE=NSIDE, results=skydict, sim=True)

    # -- Return to top level directory, continue through list of jobs
    plt.close('all')
    os.chdir(topdir)

# -- Need to set the matplotlib figure size
plt.rcParams['figure.figsize'] = 8, 6

# -------------
# Make PP plot
# ------------
injconArray = np.array(skydict['injcontour'])
print injconArray
xtics = np.arange(0,1.01,0.01)
numFound = np.array( [ (injconArray <= xvalue).sum() for xvalue in xtics] )
fracFound = 1.0*numFound / injconArray.size

plt.close('all')
plt.figure()

# -- Make 1 sigma confidence region
print "attempting to calculate PP plots"
bi = stats.binom
N = len(injconArray)

# -- 1 sigma
con = 0.68
edge = (1 - con)/2.0
uppery = bi.ppf(1-edge, N, xtics)
lowery = bi.ppf(edge, N, xtics)
uppery[0]=0
lowery[0]=0
plt.fill_between(xtics, lowery/N, uppery/N, facecolor='gray', alpha=0.2)

# -- 2 sigma
con = 0.95
edge = (1 - con)/2.0
uppery = bi.ppf(1-edge, N, xtics)
lowery = bi.ppf(edge, N, xtics)
uppery[0]=0
lowery[0]=0
plt.fill_between(xtics, lowery/N, uppery/N, facecolor='gray', alpha=0.2)


# -- 3 sigma
con = 0.997
edge = (1 - con)/2.0
uppery = bi.ppf(1-edge, N, xtics)
lowery = bi.ppf(edge, N, xtics)
uppery[0]=0
lowery[0]=0
plt.fill_between(xtics, lowery/N, uppery/N, facecolor='gray', alpha=0.2)


plt.plot(xtics, fracFound)
plt.xlabel('Confidence Region')
plt.ylabel('Fraction of injections found')
plt.plot( [0, 1], [0, 1], 'k--', linewidth=2 )
plt.savefig('pp.png')
plt.close()

# -------------------
# Histogram the areas
# -------------------
bins = np.arange(0, np.log10(41253), np.log10(41253)/20)
area50 = np.array(skydict['area50'])
area90 = np.array(skydict['area90'])
searcharea = np.array(skydict['searcharea'])
plt.figure()
plt.hist(np.log10(area50), bins=bins)
plt.xlabel('log10 Area for 50% Confidence')
plt.ylabel('Number of injections')
plt.savefig('area50.png')
plt.close()

plt.figure()
plt.hist(np.log10(area90), bins=bins)
plt.xlabel('log10 Area for 90% Confidence')
plt.ylabel('Number of injections')
plt.savefig('area90.png')
plt.close()

plt.figure()
plt.hist(np.log10(searcharea), bins=bins)
plt.xlabel('log10 Search area')
plt.ylabel('Number of injections')
plt.savefig('searcharea.png')
plt.close()


# ----------------------
# Make histogram of SNRs
# ----------------------
plt.figure()
plt.hist(snrList)
plt.xlabel('SNR')
plt.ylabel('Number')
plt.savefig('snr_hist.png')
plt.close()

# ---------------------------
# Make plot of all injections 
# ---------------------------
plt.figure()
# for sig_noise, sig_gl, snr in zip(signal, glitch, snrList):
#      plt.scatter(limit_odds(sig_gl), limit_odds(sig_noise), c=snr, s=40, cmap=matplotlib.cm.gray)

plt.scatter(glitch, signal, c=snrList, s=40)
cbar = plt.colorbar()
cbar.set_label('Network SNR', rotation=270)
plt.ylabel('log( E_signal / E_noise )')
plt.xlabel('log( E_signal / E_glitch )')
plt.axis([np.min(glitch), np.max(glitch), np.min(signal), np.max(signal)])
plt.grid()
plt.savefig('net_snr_all.png')

print np.max(signal)
print np.max(glitch)
print signal
print "Found {0} injections".format(len(glitch))

plt.axis([-50, 50, -50, 50])
plt.savefig('net_snr_zoom.png')
plt.close()


# --------------------
# Make SNR table 
# -------------------
snrTable = open('snrTable.html', 'w')
snrTable.write('<html><body>')
snrTable.write('<h1>Bayes Wave Burst Event Overviews</h1>')
snrTable.write('<table>')
snrTable.write('<tr><b><td>SNR <td>Job    <td>Evidence(S/N)  <td>Evidence(S/G)</b></tr>')

# -- Make tuple, sort by SNR
raw_eventList = zip(snrList, signal, glitch, goodJobList)
eventList =  sorted(raw_eventList, key=lambda event: event[0], reverse=True)

for (snr, sn, sg, job) in eventList:
    snrTable.write("""<tr>
<td>{0:.2f}  
<td><a href='{1}'>{1}</a>  
<td>{2:.2f}  
<td>{3:.2f}
</tr>
""".format(snr, job, sn, sg) )
snrTable.write('</table></html></body>')
snrTable.close()


# -------------------------
# Try making sky area table
# -------------------------
skyTable = open('skyTable.html', 'w')
skyTable.write('<html><body>')
skyTable.write('<h1>Bayes Wave Burst Sky Table</h1>')
skyTable.write('<img src="./pp.png" width=600>')
skyTable.write('<img src="./area50.png" width=600>')
skyTable.write('<img src="./area90.png" width=600>')
skyTable.write('<img src="./searcharea.png" width=600>')
skyTable.write('<h3>List of events</h3>')
skyTable.write('<table>')
skyTable.write('<tr><b><td>SNR <td>Job    <td>50% Area  <td>90% Area <td>Found Contour</b></tr>')

# -- Make tuple, sort by SNR
raw_eventList = zip(snrList, area50, area90, injconArray, goodJobList)
eventList =  sorted(raw_eventList, key=lambda event: event[0], reverse=True)
for (snr, a50, a90, injc, job) in eventList:
    skyTable.write("""<tr>
<td>{0:.2f}  
<td><a href='{1}/skyview.html'>{1}</a>  
<td>{2:.2f}  
<td>{3:.2f}
<td>{4:.2f}
</tr>
""".format(snr, job, a50, a90, injc) )
skyTable.write('</table></html></body>')
skyTable.close()

# -----------------------------
# -- Try making efficiency plot
# ----------------------------
glitch = np.array(glitch)
signal = np.array(signal)
scale = np.array(scale)
uniq_scale = np.array(list(set(scale)))
snrList = np.array(snrList)
eff = np.array([])
for s in uniq_scale:
    indx = [scale == s]
    this_signal = signal[indx]
    this_glitch = glitch[indx]
    total_inj = this_signal.size
    detected = np.logical_and(this_signal>0, this_glitch>0).sum()
    eff = np.append(eff, detected*1.0/total_inj)

plt.semilogx(uniq_scale, eff, 'o')
plt.ylim((0,1))
plt.xlim(( uniq_scale.min()*0.9, uniq_scale.max()*1.1 ))
plt.xlabel('Scale factor')
plt.ylabel('Detection Efficiency')
plt.grid()
plt.savefig('eff.png')
plt.close()

# -- Make plot without high SNR signals
snrmax = 50
glitch = np.array(glitch)
signal = np.array(signal)
snrList = np.array(snrList)
qglitch = glitch[ snrList < snrmax ]
qsignal = signal[ snrList < snrmax ]
qsnr    = snrList[ snrList < snrmax ] 

if len(qglitch) > 0 and len(qsignal) > 0 and len(qsnr) > 0:
    plt.figure()
    plt.scatter(qglitch, qsignal, c=qsnr, s=40)
    cbar = plt.colorbar()
    cbar.set_label('Network SNR', rotation=270)
    plt.ylabel('log( E_signal / E_noise )')
    plt.xlabel('log( E_signal / E_glitch )')
    plt.grid()
    plt.axis([np.min(qglitch)-10, np.max(qglitch)+10, np.min(qsignal)-10, np.max(qsignal)+10])
    plt.savefig('quiet.png')
    plt.close()

# -- Super low SNR
snrmax = 18
glitch = np.array(glitch)
signal = np.array(signal)
snrList = np.array(snrList)
qglitch = glitch[ snrList < snrmax ]
qsignal = signal[ snrList < snrmax ]
qsnr    = snrList[ snrList < snrmax ] 

if len(qglitch) > 0 and len(qsignal) > 0 and len(qsnr) > 0:
    plt.figure()
    plt.scatter(qglitch, qsignal, c=qsnr, s=40)
    cbar = plt.colorbar()
    cbar.set_label('Network SNR', rotation=270)
    plt.ylabel('log( E_signal / E_noise )')
    plt.xlabel('log( E_signal / E_glitch )')
    plt.grid()
    plt.axis([np.min(qglitch)-10, np.max(qglitch)+10, np.min(qsignal)-10, np.max(qsignal)+10])
    plt.savefig('super_quiet.png')
    plt.close()


# -----------------------
# Make top level web page
# ----------------------
outfile = open('top.html', 'w')
outfile.write('<html><body>')
outfile.write('<h1>BayesWave Burst Injection Run</h1>')
outfile.write("<h4><a href='./snrTable.html'>Link to the SNR table for an overview of each event.</a></h4><br/>")
outfile.write("<h4><a href='./skyTable.html'>Link to the SKY table for position reconstructions.</a></h4><br/>")
outfile.write("<h3>Efficiency Curve</h3>")
outfile.write("<img src='eff.png' alt='eff.png' width=600>")
outfile.write('<h3>All Injections</h3>')
outfile.write("<img src='net_snr_all.png' alt='net_snr_all.png' width=600>")
outfile.write("<h3>All injections, fixed axis scale</h3>")
outfile.write("<img src='net_snr_zoom.png' alt='zoom' width=600>")
outfile.write("<h3>Injections with moderate SNR</h3>")
outfile.write("<img src='quiet.png' alt='quiet' width=600>")
outfile.write("<h3>Injections with low SNR</h3>")
outfile.write("<img src='super_quiet.png' alt='super_quiet' width=600>")
outfile.write("</body></html>")
outfile.close()
