import matplotlib
matplotlib.use('Agg')
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.mlab as mlab
import glob
import argparse
import os
import pwd


# -------------------------------------------------
# Define helper function to extract median waveform
# -------------------------------------------------
def get_waveform(fileList, ifo):
    if not len(fileList) == 0:
        count=0
        for filename in fileList:
            time, strain = np.loadtxt(filename, unpack=True)
            if count == 0:
                strain_master = np.zeros((200, strain.size))
            strain_master[count] = strain
            count += 1
        print "   "+str(count)+" waveforms found for IFO "+str(int(ifo)+1)
        median_waveform = np.median(strain_master, axis=0)

        down_vec = np.zeros(median_waveform.size)
        up_vec = np.zeros(median_waveform.size)
        for sample in range(median_waveform.size):
            vector = np.transpose(strain_master)[sample]
            sort_vec = np.sort(vector)
            up_quart = sort_vec[ int(0.84*sort_vec.size) ]
            down_quart = sort_vec[ int(0.16*sort_vec.size) ]
            down_vec[sample] = down_quart
            up_vec[sample] = up_quart
            
    return (time, median_waveform, up_vec, down_vec)

# -- Extract median waveform from big output file
def get_waveform_bigfile(filename, ifo):
    strain_master = np.loadtxt(filename)
    median_waveform = np.median(strain_master, axis=0)
    down_vec = np.zeros(median_waveform.size)
    up_vec = np.zeros(median_waveform.size)
    for sample in range(median_waveform.size):
        vector = np.transpose(strain_master)[sample]
        sort_vec = np.sort(vector)
        up_quart = sort_vec[ int(0.84*sort_vec.size) ]
        down_quart = sort_vec[ int(0.16*sort_vec.size) ]
        down_vec[sample] = down_quart
        up_vec[sample] = up_quart
            
    return (median_waveform, up_vec, down_vec)

def get_median(strain_master, ifo):    
    median_waveform = np.median(strain_master, axis=0)
    down_vec = np.zeros(median_waveform.size)
    up_vec = np.zeros(median_waveform.size)
    for sample in range(median_waveform.size):
        vector = np.transpose(strain_master)[sample]
        sort_vec = np.sort(vector)
        up_quart = sort_vec[ int(0.84*sort_vec.size) ]
        down_quart = sort_vec[ int(0.16*sort_vec.size) ]
        down_vec[sample] = down_quart
        up_vec[sample] = up_quart            
    return (median_waveform, up_vec, down_vec)


# ------------------------------------------------------------
# Calculate the mode and confidence interval from a histogram
# ------------------------------------------------------------
def get_mode(n, bins):
    
    # -- Take average of bins to get rid of size mismatch
    if (bins.size) == (n.size + 1):
        new_bins = [0.5*(bins[i] + bins[i+1]) for i in range(0,n.size)]
        bins = new_bins

    # -- Set the index of the mode to the tallest peak
    modeindx = n.argmax()
    samps_90 = 0.90*n.sum()
    
    for i in np.arange(n.size):
        lowindx = np.max([0, modeindx-i])
        highindx = np.min([n.size-1, modeindx+i])
        n_inrange = n[lowindx:highindx+1]
        if n_inrange.sum() >= samps_90:
            low = bins[lowindx]
            high = bins[highindx]
            mode = bins[modeindx]
            break
    return(mode, low, high)
    

    
# -- Create the Bayesogram
def bayesogram(filename, time, twodrange):
    strain_master = np.loadtxt(filename)
    multiply = strain_master.size / time.size
    X = list(time)*multiply
    Y = strain_master.flatten()
    # twodrange = [timerange, [Y.min(), Y.max()]]
    timerange = twodrange[0]
    Nx = np.abs( (timerange[1]-timerange[0])/(time[1] - time[0]) )
    H, xedges, yedges = np.histogram2d(X,Y,bins=[Nx, 100],range=twodrange)

    # H needs to be rotated and flipped
    H = np.rot90(H)
    H = np.flipud(H)
  
    # Plot 2D histogram using pcolor
    fig2 = plt.figure()
    plt.pcolormesh(xedges,yedges,H)
    # cbar = plt.colorbar(orientation='horizontal')
    # cbar.ax.set_ylabel('Counts')
    return fig2

    for sample in range(median_waveform.size):
        vector = np.transpose(strain_master)[sample]
        sort_vec = np.sort(vector)
        up_quart = sort_vec[ int(0.84*sort_vec.size) ]
        down_quart = sort_vec[ int(0.16*sort_vec.size) ]
        down_vec[sample] = down_quart
        up_vec[sample] = up_quart
            
    return (median_waveform, up_vec, down_vec)



# -- Parse command line arguments
parser = argparse.ArgumentParser(description='Produce html page for a BayesWaveBurst run.')
parser.add_argument('--mdc', default='True', help='Is this this an MDC? True/False (Default is false)')
args = parser.parse_args()
mdc = args.mdc

# ---------------------------------------
# Extract information from bayeswave.run
# ---------------------------------------
jobname = glob.glob('*bayeswave.run')[0]
bayeswave = open(jobname, 'r')

ifoNames = []
for line in bayeswave:
    spl = line.split()
    if len(spl) < 1: continue
    del spl[-1]
    # Determine names of IFOs
    if spl[0] == 'Command' and spl[1] == 'line:':
        for index, splElement in enumerate(spl):
            if splElement == '--ifo':
                ifoNames.append(spl[index+1]) 
        continue         
    # Determine number of IFOs
    if ' '.join(spl) == 'number of detectors':
        spl = line.split()
        ifoNum = spl[3]
        continue         
    # Determine gps trigger time
    if ' '.join(spl) == 'trigger time (s)':
        spl = line.split()
        gps = spl[3]
        break
ifoNum = int(ifoNum)
if ifoNum == 1:
    ifoList = ['0']
elif ifoNum == 2:
    ifoList = ['0', '1']
elif ifoNum == 3:
    ifoList = ['0', '1', '2']

# ----------------------------------
# Determine name of job 
# ----------------------------------
jobname = jobname.split('_')
del jobname[-1]
if not len(jobname) == 0:
    jobname = '_'.join(jobname)+'_'
else:
    jobname = ''

# -------------------------------------------------
# Read in data and median waveform to get plot axis
# -------------------------------------------------
axisList = []
# Determine the last data frame filename: avoids assuming it is *_199.dat.*
frame_list = os.listdir('waveforms/')
data_frame_list = frame_list
for file in frame_list[:]:
    if not('clean_1_data_' in file and '.dat.0' in file):
        data_frame_list.remove(str(file))
last_data_frame = str(data_frame_list[-1].split('.0')[0])

# -- Read in time vector
time = np.loadtxt('timesamp.txt')

for ifo in ifoList:
    # -- Read Data
    #d_t, d_s = np.loadtxt('waveforms/'+str(jobname)+'clean_1_data_199.dat.{0}'.format(ifo), unpack=True)
    d_t, d_s = np.loadtxt('waveforms/'+last_data_frame+'.{0}'.format(ifo), unpack=True)

    # -- Read Signal model
    filename = 't_recOut.dat.{0}'.format(ifo)
    median_waveform, up_vec, down_vec = get_waveform_bigfile(filename, ifo)

    # fileList = glob.glob('waveforms/'+str(jobname)+'signal_1_wave_*.dat.{0}'.format(ifo))
    # (time, median_waveform, up_vec, down_vec) = get_waveform(fileList, ifo)
    
    # -- Get axis info
    wave = up_vec
    # wave = median_waveform
    # wave_max = np.maximum(wave.max(), d_s.max())
    wave_max = wave.max()
    sig_times = time[wave > 0.05*wave.max()]
    axisList.append([sig_times.min(), sig_times.max(), -wave_max*1.1, wave_max*1.1])

# -- Select Axis with biggest y-value
ymax = 0
for axcand in axisList:
    if axcand[3] > ymax:
        ymax = axcand[3]
        axwinner = axcand

print "   The axis winner is", axwinner

for ifo in ifoList:

    # ----------------------------------
    # Read in signal model and data TODO: CHECK IF THE FIRST 2 LINES ARE REDUNDANT 
    # ----------------------------------
    # fileList = glob.glob('waveforms/'+str(jobname)+'signal_1_wave_*.dat.{0}'.format(ifo))
    # (time, median_waveform, up_vec, down_vec) = get_waveform(fileList, ifo)
    # print time, median_waveform, "\n"

    filename = 't_recOut.dat.{0}'.format(ifo)
    median_waveform, up_vec, down_vec = get_waveform_bigfile(filename, ifo)
    time = np.loadtxt('timesamp.txt')

    # --- Read in the injected waveform
    filename = 't_injOut.dat.{0}'.format(ifo)
    inj_median_waveform, inj_up_vec, inj_down_vec = get_waveform_bigfile(filename, ifo)    
    inj_time = time

    # -- Read whitened data
    #d_t, d_s = np.loadtxt('waveforms/'+str(jobname)+'clean_1_data_199.dat.{0}'.format(ifo), unpack=True)
    d_t, d_s = np.loadtxt('waveforms/'+last_data_frame+'.{0}'.format(ifo), unpack=True)
  
    # -- Plot the median waveform and injected waveform
    fignum = plt.figure()
    # plt.plot(d_t, d_s, 'r', linewidth=1, alpha=0.3)
    plt.plot(time, median_waveform, 'b', linewidth=2)
    plt.plot(time, inj_median_waveform, 'r', linewidth=1)

    # plt.fill_between(np.arange(median_waveform.size), down_vec, up_vec, facecolor='blue', edgecolor='blue', alpha=0.2)
    plt.fill_between(time, down_vec, up_vec, facecolor='blue', edgecolor='blue', alpha=0.2)
    plt.fill_between(time, inj_down_vec, inj_up_vec, facecolor='red', edgecolor='red', alpha=0.2)
    
    # -- Output file for Tyson
    tyson_data = np.vstack((time, median_waveform, down_vec, up_vec, inj_median_waveform, inj_down_vec, inj_up_vec))
    np.savetxt('med_waveform_{0}.txt'.format(ifo),  np.transpose(tyson_data))

    plt.axis(axwinner)
    axisList.append([sig_times.min(), sig_times.max(), -wave_max*1.1, wave_max*1.1])
    plt.xlabel('Time (s)')
    plt.ylabel('Best Fit Waveform (blue) and injected waveform (red)')
    plt.title('IFO {0}'.format(ifo))

    # -- Save the plot
    plt.savefig('pec_waveform_{0}.png'.format(ifo))
    plt.close()


    # -- Plot the median spectrum
    # spec_master = np.loadtxt('f_recOut.data.{0}'.format(ifo))
    # mag_master  = [ (a[0]**2 + a[1]**2) for a in 
    # med_spec, high_spec, low_spec = get_median(spec_master, ifo)

    # -- Bayesogram 
    filename = 't_recOut.dat.{0}'.format(ifo)
    fig2 = bayesogram(filename, time, [axwinner[0:2], axwinner[2:4]])
    plt.plot(time, median_waveform, 'w', linewidth=1, alpha=0.5)
    plt.axis(axwinner)
    plt.xlabel('Time (s)')
    plt.ylabel('h(t)')
    plt.title('IFO {0}'.format(ifo))
    plt.savefig('t_bgram_{0}.png'.format(ifo))
    plt.close()
    
    # -- Attempt to make a spectrogram of median waveform
    ts = time[1] - time[0]
    fs = int(np.round(1.0/ts))
    NFFT = fs / 16
    window = np.blackman(NFFT)
    holder = np.zeros(1*NFFT)
    seg = np.append(holder, median_waveform)
    seg = np.append(seg, holder)
    spec_power, freqs, bins = mlab.specgram(median_waveform, NFFT=NFFT, Fs=fs, 
                                            window=window, noverlap=int(0.8*NFFT))

    # -- Try eliminating quiet pixels
    sort_power = np.sort(spec_power.flatten())[::-1]
    sum_power = np.cumsum(sort_power)
    indx = np.nonzero( sum_power < 0.95*sum_power.max() )
    point90 = sort_power[indx[0].max()]
    spec_power[ spec_power < point90 ] = spec_power.min()
    vmin = np.log10(spec_power[ spec_power > spec_power.min()].min())

    # -- Try do it yourself spectrogram
    fig = plt.figure()
    ax = plt.subplot(111)
    binwidth = (bins[1] - bins[0]) / 2.0
    cax = ax.pcolormesh(bins-0.5-binwidth, freqs, np.log10(spec_power), vmin=vmin, cmap='OrRd')
    plt.title('IFO {0}'.format(ifo))
    plt.xlabel('Time (s)')
    plt.ylabel('Frequency (Hz)')
    plt.axis([axwinner[0], axwinner[1], 0, fs/2])
    cb = plt.colorbar(cax, orientation='horizontal')
    cb.set_label('Logrithmic Signal Power')
    plt.savefig('self_spec_{0}.png'.format(ifo))
    plt.close()

    # --------------------------------
    # -- Plot the median glitch model
    # --------------------------------
    # fileList = glob.glob('waveforms/'+str(jobname)+'glitch_1_glitch_*.dat.{0}'.format(ifo))
    # (time, median_waveform, up_vec, down_vec) = get_waveform(fileList, ifo)

    # plt.figure()
    # plt.plot(d_t, d_s, 'r', linewidth=1, alpha=0.3)
    # plt.plot(time, median_waveform, 'k', linewidth=2)
    # plt.fill_between(time, down_vec, up_vec, facecolor='black', edgecolor='black', alpha=0.3)
    
    # -- Adjust plot properties
    # plt.axis(axwinner)
    # plt.xlabel('Time (s)')
    # plt.ylabel('Best Fit Glitch Model (whitened)')
    # plt.title('IFO {0}'.format(ifo))

    # -- Save the plot
    # plt.savefig('glitch_{0}.png'.format(ifo))
    # plt.close()

    # ---------------------------------
    # Make histograms of PEC parameters
    # ---------------------------------

    for white in [False, True]:

        # -- Read moments file and open mode output file
        if white:
            moments = np.recfromtxt('w_moments.dat.{0}'.format(ifo), names=True)
            outfile = open('w_mode_{0}.txt'.format(ifo), 'w')
        else:
            moments = np.recfromtxt('moments.dat.{0}'.format(ifo), names=True)
            outfile = open('mode_{0}.txt'.format(ifo), 'w')
        
        moments = moments[len(moments)/2:]

        outfile.write("# columns are the mode, followed by the lower and upper bounds of the 90% confidence interval\n")
        outfile.write("# Rows are: Overlap, log10(Energy), t0, log10(duration), f0, bandwidth\n")
        outfile.write("# Recovered values are followed by injected values\n")


        # -- Histogram overlaps
        plt.figure()

        overlap = moments['overlap']
        bins = np.arange(-1.02,1.02,0.01)
        n,bins,patches = plt.hist(overlap, bins, alpha=0.5, log=True, linewidth=0)
        plt.grid()
        plt.xlim(-1.1, 1.1)
        plt.xlabel('Overlap - recovered signal and injection')
        if white:
            plt.savefig('woverlap_{0}.png'.format(ifo))
        else:
            plt.savefig('overlap_{0}.png'.format(ifo))
        plt.close()
        
        mode, low, high = get_mode(n, bins)
        outfile.write("{0} {1} {2} \n".format(mode, low, high))
    
        # -- Histogram of signal energy
        plt.figure()

        rn,rbins,patches = plt.hist(np.log10(moments['energy_rec']), bins=100, label='Recovered', alpha=0.5, log=True, linewidth=0)
#        n,bins,patches = plt.hist(np.log10(moments['energy_inj']), bins=rbins, alpha=0, color='yellow', log=True)
        inmode = np.median(np.log10(moments['energy_inj']))
        mode, low, high = get_mode(rn, rbins)
        outfile.write("{0} {1} {2} \n".format(mode, low, high))
        # inmode, low, high = get_mode(n, bins)
        outfile.write("{0} {1} {2} \n".format(inmode, low, high))

        plt.axvline(x=inmode, label='Injected', color='r')
        plt.xlabel("log10 (Signal Energy)")
        plt.title('IFO {0}'.format(ifo))
        plt.grid()
        plt.ylim(1, 2*rn.max())
        # bindx = np.append(rn > 1, False)    This line is probably junk
        plt.legend()
        if white:
            plt.savefig('wenergy_{0}.png'.format(ifo))
        else:
            plt.savefig('energy_{0}.png'.format(ifo))
        plt.close()

        print "Got log10(energy) of {0}, with range {1} to {2}".format(mode, low, high)

        # -- Histogram central time
        plt.figure()
        rn,rbins,patches = plt.hist(moments['time_rec'] - 2, bins=100, label='Recovered', alpha=0.5, log=True, linewidth=0)
        # n,bins,patches = plt.hist(moments['time_inj'] - 2, bins=rbins, alpha=0, color='yellow', log=True)
        inmode = np.median(moments['time_inj'] - 2 )
        mode, low, high = get_mode(rn, rbins)
        outfile.write("{0} {1} {2} \n".format(mode, low, high))        
        outfile.write("{0} {1} {2} \n".format(inmode, low, high))
        plt.axvline(x=inmode, label='Injected', color='r')

        plt.xlabel("Central Time")
        plt.title('IFO {0}'.format(ifo))
        plt.ylim(1, 2*rn.max())
        plt.legend()
        plt.grid()
        if white:
            plt.savefig('wtzero_{0}.png'.format(ifo))
        else:
           plt.savefig('tzero_{0}.png'.format(ifo))
        plt.close()

        # -- Histogram of duration
        plt.figure()
        rn,rbins,patches = plt.hist(np.log10(moments['duration_rec']), bins=100, label='Recovered', alpha=0.5, log=True, 
                                    linewidth=0)
        # n,bins,patches = plt.hist(np.log10(moments['duration_inj']), bins=rbins, alpha=0.0, color='yellow', 
        #                           log=True)
        # inmode, low, high = get_mode(n, bins)
        inmode = np.median(np.log10(moments['duration_inj']))
        mode, low, high = get_mode(rn, rbins)
        outfile.write("{0} {1} {2} \n".format(mode, low, high))
        outfile.write("{0} {1} {2} \n".format(inmode, low, high))
        plt.axvline(x=inmode, label='Injected', color='r')

        plt.xlabel("log10(Duration)")
        plt.title('IFO {0}'.format(ifo))
        plt.ylim(1, 2*rn.max())
        plt.legend()
        plt.grid()
        if white:
            plt.savefig('wduration_{0}.png'.format(ifo))
        else:
            plt.savefig('duration_{0}.png'.format(ifo))
        plt.close()

        # -- Histogram of central frequency
        plt.figure()
        rn,rbins,patches = plt.hist(moments['f_rec'], bins=100, label='Recovered', alpha=0.5, linewidth=0, log=True)
        # n,bins,patches = plt.hist(moments['f_inj'], bins=rbins, alpha=0.0, color='yellow', log=True)
        inmode = np.median(moments['f_inj'])
        mode, low, high = get_mode(rn, rbins)
        outfile.write("{0} {1} {2} \n".format(mode, low, high))
        # inmode, low, high = get_mode(n, bins)
        outfile.write("{0} {1} {2} \n".format(inmode, low, high))
        plt.axvline(x=inmode, label='Injected', color='r')

        plt.xlabel("Central Freq (Hz)")
        plt.title('IFO {0}'.format(ifo))
        plt.ylim(1, 2*rn.max())
        plt.legend()
        plt.grid()
        if white:
            plt.savefig('wfreq_{0}.png'.format(ifo))
        else:
            plt.savefig('freq_{0}.png'.format(ifo))
        plt.close()
        print "Got Central Freq. {0}, with range {1} to {2}".format(mode, low, high)

        # -- Histogram of bandwidth
        plt.figure()
        rn,rbins,patches = plt.hist(moments['ban_rec'], bins=100, label='Recovered', alpha=0.5, linewidth=0, log=True)
       # n,bins,patches = plt.hist(moments['band_inj'], bins=rbins, alpha=0, color='yellow', log=True)
        inmode = np.median(moments['band_inj'])

        mode, low, high = get_mode(rn, rbins)
        outfile.write("{0} {1} {2} \n".format(mode, low, high))
        # inmode, low, high = get_mode(n, bins)
        outfile.write("{0} {1} {2} \n".format(inmode, low, high))
        plt.axvline(x=inmode, label='Injected', color='r')

        plt.xlabel("Bandwidth (Hz)")
        plt.title('IFO {0}'.format(ifo))
        plt.ylim(1, 2*rn.max())
        plt.grid()
        plt.legend()
        if white:
            plt.savefig('wband_{0}.png'.format(ifo))
        else:
            plt.savefig('band_{0}.png'.format(ifo))
        plt.close()
       
        # --------------------------
        # For debugging, plot signal energy
        # ----------------------
        plt.figure()
        e_freq, energy = np.loadtxt('rho.txt', unpack=True)
        plt.plot(e_freq,energy)
        plt.grid()
        plt.ylabel('Signal energy')
        plt.xlabel('Frequency')
        plt.savefig('sig_energy.png')


        # --------------------------
        # For debugging, plot signal energy (time domain)
        # ----------------------
        plt.figure()
        t, e = np.loadtxt('t_rho.txt', unpack=True)
        plt.plot(t,e)
        plt.grid()
        plt.ylabel('Signal energy')
        plt.xlabel('Time')
        plt.xlim(1.98, 2.02)
        plt.savefig('t_sig_energy.png')


        # --------------------------
        # For debugging, plot time domain waveform
        # ----------------------
        plt.figure()
        t, h = np.loadtxt('sanity_time.txt', unpack=True)
        plt.plot(t,h)
        plt.grid()
        plt.xlim(1.98, 2.02)
        plt.ylabel('Waveform')
        plt.xlabel('Time')
        plt.savefig('hoft.png')
        
        # -- End the loop over white/colored moments
        outfile.close()

    # -----------------------
    # Plot the median PSDs
    # -----------------------
    fileList = glob.glob('waveforms/'+str(jobname)+'clean_1_psd_*.dat.{0}'.format(ifo))

    if not len(fileList) == 0:
        count=0
        for filename in fileList:
            freq, power, spline, line = np.loadtxt(filename, unpack=True)
            if count == 0:
                power_master = np.zeros((200, power.size))
                spline_m = np.zeros((200, power.size))
                line_m = np.zeros((200, power.size))

            power_master[count] = power
            spline_m[count] = spline
            line_m[count] = line

            count += 1

        median_power = np.median(power_master, axis=0)
        median_line = np.median(line_m, axis=0)
        median_spline = np.median(spline_m, axis=0)

        plt.figure()
        plt.loglog(freq, median_power,'b', alpha=0.4, label='PSD of Noise')
        plt.loglog(freq, median_line + median_spline, 'r', label='Noise model')
        plt.xlabel('Frequency')
        plt.ylabel('PSD')
        plt.title('PSD for IFO {0}'.format(ifo))
        plt.xlim([freq.min(), freq.max()])
        plt.grid()
        plt.legend()
        plt.savefig('psd_{0}.png'.format(ifo))
        plt.close()

# --------------
# Plot Evidence
# --------------

maxodds=20

infile = open(str(jobname)+'evidence.dat', 'r')
maxodds = 20
for line in infile:
    spl = line.split()
    if spl[0] == 'noise':
        sig_noise = -float(spl[1])
    if spl[0] == 'glitch':
        sig_gl = -float(spl[1])
infile.close()

print
print '   log( E_signal / E_noise ) =', sig_noise
print '   log( E_signal / E_glitch ) =', sig_gl

plt.figure()
plt.plot(sig_gl, sig_noise, '*', markersize=15, mfc='yellow')
plt.ylabel('log( E_signal / E_noise )')
plt.xlabel('log( E_signal / E_glitch )')

maxodds = 1.1*np.array( [np.abs(sig_gl), np.abs(sig_noise), 20] ).max()
xmaxodds = 1.1*np.maximum(np.abs(sig_gl), 20)
ymaxodds = 1.1*np.maximum(np.abs(sig_noise), 20)

plt.axis([-xmaxodds, xmaxodds, -ymaxodds, ymaxodds])

plt.fill_between([0,maxodds], [0, 0], [maxodds, maxodds], facecolor='green', interpolate=True, alpha=0.3)
plt.fill_between([-maxodds,0,maxodds], [-maxodds,0,0], [-maxodds, -maxodds, -maxodds], facecolor='blue', interpolate=True, alpha=0.3)
plt.fill_between([-maxodds,0], [-maxodds, 0], [maxodds, maxodds], facecolor='red', interpolate=True, alpha=0.3)

plt.grid()

plt.text(0.9*xmaxodds, 0.9*ymaxodds, 'Signal',
        horizontalalignment='right',
        verticalalignment='top')

plt.text(-0.9*xmaxodds, 0.9*ymaxodds, 'Glitch',
        horizontalalignment='left',
        verticalalignment='top')

plt.text(0.9*xmaxodds, -0.9*ymaxodds, 'Noise',
        horizontalalignment='right',
        verticalalignment='bottom')

plt.savefig('odds.png')
plt.close()

# ----------------
# Plot Temp Chains
# ---------------
modelList = ('glitch', 'signal', 'noise')

# -- Make first plot
plt.figure()
for model in modelList:
    temp, likehood, error = np.recfromtxt(str(jobname)+"{0}_1_evidence.dat".format(model), unpack=True)
    plt.semilogx(temp, likehood, label=model, linewidth=2)
plt.xlabel( '1/Temp' )
plt.ylabel('log(L)')
plt.grid()
plt.legend(loc=2)
plt.savefig('likelihood.png'.format(model))
plt.close()

# -- Make 2nd plot
plt.figure()
for model in modelList:
    temp, likehood, error = np.recfromtxt(str(jobname)+"{0}_1_evidence.dat".format(model), unpack=True)
    plt.semilogx(temp, likehood*temp, linewidth=2, label=model)
plt.grid()
plt.xlabel('1/Temp')
plt.ylabel('log(L) X 1/Temp')
plt.legend(loc=2)
plt.savefig('TL.png'.format(model))
plt.close()

# ---------------------
# Make web page
# ---------------------
web = open('pec.html', 'w')
web.write('<html><body>\n')

# -- Get some text from condor out: if condorOut.txt is not there, we will use the info from bayeswave.run
info = ""
if os.path.isfile('condorOut.txt'):
    condor = open('condorOut.txt', 'r')
    ifoNames = []
    for line in condor:
        spl = line.split()
        if len(spl) < 1: continue
        if spl[0] == 'Detector':
            ifoNames.append(spl[1].strip(':'))
        if spl[0] == 'trigger':
            gps = float(spl[3])
        if line.find('Run Flags') > 0:
            break
    #    if line.find('Run Flags') > 0:
    #        break
    #    else:
    #        info = info + line
    condor.close()

# -- If MDC, read SNR info
if mdc=='True':
    print "Looking for SNRs..."
    snrList = []
    foundSnr=False
    snrFile = open('condorOut.txt', 'r')
    for line in snrFile:
        if line.find('Recompute')>0:
            print "Found Recompute section!"
            foundSnr = True
        if (line.find('Injected SNR')>-1) and foundSnr:
            print "Found an SNR!!"
            snrList.append(float(line.split()[2]))
    snrFile.close


info = info + "The detectors are {0} \n".format( ', '.join(ifoNames) )
info = info + "   The trigger time is GPS {0} \n".format(gps)

# -- Start the web page
web.write('<h1>Bayeswave Burst Output page</h1>\n')
web.write("<h3>Results for trigger at GPS {0}</h3>".format(gps) )
web.write("<h3>Detector names: {0}</h3>".format(', '.join(ifoNames) ) )

# -- Print SNRs if MDC
if mdc=='True':
    web.write("<h3>Matched Filter SNRs of injections</h3>")
    for ifo, snr in zip(ifoNames, snrList):
        web.write("{0} injected with SNR {1:.3f}<br/>".format(ifo, snr))

# -- Write out the basic log info
web.write('<h3>Log Info</h3>')
# web.write("<pre>{0}</pre>".format(info))
web.write("<a href='./condorOut.txt'>See full log file</a>")
print "   Info: {0}".format(info)


# -- Write out the bayes Factor plot
web.write('<h3>Evidence for Signal</h3>')
web.write(" log(Evidence_signal / Evidence_glitch) = {0:.2f} <br/>".format(sig_gl) )
web.write(" log(Evidence_signal / Evidence_noise) = {0:.2f} <br/>".format(sig_noise) )
web.write("<img src='./odds.png' width=600 alt='odds.png'>")

# -- Write out the waveforms
web.write("<h3>Median Signal Model Waveforms and 1-sigma Uncertainties</h3>")
web.write("Reconstructed waveforms in whitened data.  Red is injected waveform, blue is recovered waveform<br/>")
for ifo in ifoList:
    web.write("<img src='./pec_waveform_{0}.png' alt='waveform_{0}.png' width=500>".format(ifo))

# -- Write out the bayesograms in time
web.write("<h3>Bayesogram (time): 2-D Histogram of Posterior</h3>")
web.write("Color scale is histogram of posterior, trace is median reconstructed waveform<br/>")
for ifo in ifoList:
    web.write("<img src='./t_bgram_{0}.png' alt='waveform_{0}.png' width=500>".format(ifo))

# -- Here come the plots in colored noise
web.write("<h2>Plots in colored data</h2>") 

# -- Add histogram of overlaps
web.write("<h3> Overlap Histogram between recovered signal and injection</h3>")
for ifo in ifoList:
    web.write("<img src='overlap_{0}.png' alt='overlap' width=500>".format(ifo))

# -- Add histogram of signal energy
web.write("<h3>Histogram of signal energy, (aka ||<i>h(t)</i>||<sup>2</sup>) </h3>")
for ifo in ifoList:
    web.write("<img src='energy_{0}.png' alt='t0' width=500>".format(ifo))

# -- Add histogram of recovered times
web.write("<h3>Histogram of central time</h3>")
for ifo in ifoList:
    web.write("<img src='tzero_{0}.png' alt='t0' width=500>".format(ifo))

# -- Add histogram of duration
web.write("<h3>Histogram of duration</h3>")
for ifo in ifoList:
    web.write("<img src='duration_{0}.png' alt='t0' width=500>".format(ifo))

# -- Add histogram of central frequency
web.write("<h3>Histogram of central frequency</h3>")
for ifo in ifoList:
    web.write("<img src='freq_{0}.png' alt='t0' width=500>".format(ifo))

# -- Add histogram of bandwidth
web.write("<h3>Histogram of bandwidth</h3>")
for ifo in ifoList:
    web.write("<img src='band_{0}.png' alt='t0' width=500>".format(ifo))

# -- Here come the plots in colored noise
web.write("<h2>Plots in whitened data</h2>") 

# -- Add histogram of overlaps
web.write("<h3> Overlap Histogram between recovered signal and injection</h3>")
for ifo in ifoList:
    web.write("<img src='woverlap_{0}.png' alt='overlap' width=500>".format(ifo))

# -- Add histogram of signal energy
web.write("<h3>Histogram of signal energy, (aka ||<i>h(t)</i>||<sup>2</sup>) </h3>")
for ifo in ifoList:
    web.write("<img src='wenergy_{0}.png' alt='t0' width=500>".format(ifo))

# -- Add histogram of recovered times
web.write("<h3>Histogram of central time</h3>")
for ifo in ifoList:
    web.write("<img src='wtzero_{0}.png' alt='t0' width=500>".format(ifo))

# -- Add histogram of duration
web.write("<h3>Histogram of duration</h3>")
for ifo in ifoList:
    web.write("<img src='wduration_{0}.png' alt='t0' width=500>".format(ifo))

# -- Add histogram of central frequency
web.write("<h3>Histogram of central frequency</h3>")
for ifo in ifoList:
    web.write("<img src='wfreq_{0}.png' alt='t0' width=500>".format(ifo))

# -- Add histogram of bandwidth
web.write("<h3>Histogram of bandwidth</h3>")
for ifo in ifoList:
    web.write("<img src='wband_{0}.png' alt='t0' width=500>".format(ifo))

# -- Write out Glitch model
# web.write("<h3>Median Glitch Model Waveforms and 1-sigma Uncertainties</h3>")
# web.write("Reconstructed waveforms in whitened data<br/>")
# for ifo in ifoList:
#    web.write("<img src='./glitch_{0}.png' alt='glitch_{0}.png' width=500>".format(ifo))

# -- Write out spectrogram
# web.write("<h3>Spectrogram of median reconstructed signal model waveform</h3>")
# for ifo in ifoList:
#    web.write("<img src='./self_spec_{0}.png' alt='self_spec_{0}.png' width=500>".format(ifo) )

# -- Write out the PSDs
web.write("<h3>PSDs and Reconstructed PSDs</h3>")
for ifo in ifoList:
    web.write("<img src='./psd_{0}.png' alt='psd_{0}.png' width=500>".format(ifo))

# -- Write out diagnostic plots
web.write("<h3>Diagnostic plots of Likelihood and Temperature</h3>")
web.write("<img src='./likelihood.png' alt='likelihood.png' width=500>")
web.write("<img src='./TL.png' alt='TL.png' width=500> <br/>" )
    
# -- Include skymap
web.write('<h3>Skymap</h3>')
web.write("<img src='./skymap.png' alt='No_skymap_available' width=650>")

#-- Close the web site
web.write('</body></html>')
web.close()


